// ConnectVPN.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "LeoHelpers_Scope.h"

bool IsConnected(const wchar_t *szProgName, const wchar_t *szConnectionName)
{
	// RasEnumConnections enumerates active connections; disconnected ones are not listed at all.

	DWORD dwCb = 0;
	DWORD dwConnections = 0;

	if (ERROR_BUFFER_TOO_SMALL != ::RasEnumConnections(NULL, &dwCb, &dwConnections))
	{
		return false; // No connections or an error. Not connected, either way.
	}

	// The OS returns the buffer sized based on the largest RASCONN structure it knows about.
	// If we are compiled with an SDK from a later OS version that has a larger RASCONN structure
	// then the OS's suggested buffer size will not be large enough. So allocate the maximum of
	// what the OS tells us and what is actually needed.
	if (dwCb < (dwConnections * sizeof(RASCONN)))
	{
		dwCb = (dwConnections * sizeof(RASCONN));
	}

	LeoHelpers::AutoBuffer< RASCONN > rasCons;
	if (!rasCons.AllocateBytes(dwCb))
	{
		return false;
	}

	// First RASCONN structure must have its size set.
	// (And this may be larger than the OS's RASCONN size! See above.)
	rasCons.GetBuffer()[0].dwSize = sizeof(RASCONN);

	dwConnections = 0;
	
	if (ERROR_SUCCESS != ::RasEnumConnections(rasCons.GetBuffer(), &dwCb, &dwConnections))
	{
		// BUGBUG: This could fail due to a race condition in RasEnumConnections, i.e. if another connection arrived
		// between asking for the buffer size and asking for the buffer to be filled in. If this code were more critical
		// then it would make sense to loop and try again a few times on ERROR_BUFFER_TOO_SMALL...
		// ...I can't be bothered, though. :) It'd easy to code but difficult to actually test the situation.
		return false;
	}

	for (DWORD i = 0; i < dwConnections; ++i)
	{
		if (0 == _wcsicmp(szConnectionName, rasCons.GetBuffer()[i].szEntryName))
		{
			for (int i = 0; i < 320; ++i) // 320 x 500 ms => maximum 2 minute wait before giving up.
			{
				RASCONNSTATUS rasConStatus = {0};
				rasConStatus.dwSize = sizeof(RASCONNSTATUS);
				
				if (ERROR_SUCCESS != ::RasGetConnectStatus(rasCons.GetBuffer()[i].hrasconn, &rasConStatus))
				{
					return false;
				}

				if (rasConStatus.rasconnstate == RASCS_Connected)
				{
					return true;
				}

				// 08/Jul/2010: Line below had a copy & paste error befor:
			//	if (RASCS_Disconnected == RASCS_Disconnected || rasConStatus.dwError != 0)
				if (rasConStatus.rasconnstate == RASCS_Disconnected || rasConStatus.dwError != 0)
				{
					return false;
				}

				// Print after we've waited about 15 seconds.
				// (Don't want a command prompt to appear unless we're delaying for a long time. Don't expect
				// this to happen outside of exceptional situations. If this happens regularly for anyone it
				// might be worth showing a nice dialog instead of using wprintf.)
				if (i == 30)
				{
					wprintf(L"%s: Waiting for existing connection to finish...\n", szProgName);
				}

				Sleep(500); // Presumably something else is in the process of establishing the connection. Wait and loop.
			}
		}
	}

	return false; // What we were looking for isn't there yet.
}


int wmain(int argc, WCHAR * argv[])
{
	if (argc == 0)
	{
		return -1;
	}

	const wchar_t *szProgName = argv[0];

	if (argc != 2)
	{
		wprintf(L"Usage: %s <name of RAS/VPN connection>\n\nIf the connection is already active then nothing happens.\nIf the connection is inactive the standard connection dialog will be displayed.\n\nUpdates and source code: www.pretentiousname.com\n\n", szProgName);
		return -1;
	}

	const wchar_t *szConnectionName = argv[1];

	if (IsConnected(szProgName, szConnectionName))
	{
		return 0; // nothing for us to do.
	}

	// The connection is not connected (or doesn't even exist).

	RASDIALDLG rdd = {0};
	rdd.dwSize = sizeof(rdd);

	if (!::RasDialDlg(NULL, const_cast< wchar_t * >( szConnectionName ), NULL, &rdd))
	{
		// Don't expect this to happen outside of exceptional situations. If this happens
		// regularly for anyone it might be worth showing a nice dialog instead of using wprintf.)

		wprintf(L"%s: Failed to connect to %s (%lu)\n", szProgName, szConnectionName, rdd.dwError);
		return -1;
	}

	return 0;
}
